@@ -43,6 +43,7 @@ from oslo_utils import timeutils
 import six
 import six.moves.xmlrpc_client as xmlrpclib
 
+ipaddress = importutils.try_import("ipaddress")
 netaddr = importutils.try_import("netaddr")
 
 _nasty_type_tests = [inspect.ismodule, inspect.isclass, inspect.ismethod,
@@ -109,6 +110,11 @@ def to_primitive(value, convert_instances=False, convert_datetime=True,
     if netaddr and isinstance(value, netaddr.IPAddress):
         return six.text_type(value)
 
+    if ipaddress and isinstance(value,
+                                (ipaddress.IPv4Address,
+                                 ipaddress.IPv6Address)):
+        return six.text_type(value)
+
     # value of itertools.count doesn't get caught by nasty_type_tests
     # and results in infinite loop when list(value) is called.
     if type(value) == itertools.count:
@@ -126,7 +132,7 @@ def to_primitive(value, convert_instances=False, convert_datetime=True,
         return 'mock'
 
     if level > max_depth:
-        return '?'
+        return None
 
     # The try block may not be necessary after the class check above,
     # but just in case ...
